-- File: init.lua
-- Author: Martin Burgemeister, Mathis Lindemann
-- Initialize all lua functions and objects.

-- The Lua Controller
Search = LuaSearch(searchcontrol)
-- The Table Model
Table = LuaTable(tablecontrol)
-- The Controller
Output = LuaOutput(controller)
-- The Sort Controller
Sort = LuaSort(sortcontrol)

-- Get Function names
function functions()	
	local a = ""
	for key,value in pairs(_G) do
	    a = a .. key .. "\n"
	end
	return a
end

-- Search function
function search(...)
	if(arg[1] == "-h") then
		print(help["search"])
		return
	end
	
	if(arg["n"] > 1) then
		Search:search(arg[1],arg[2])
	else
		Search:search(arg[1])
	end
	numberOfHits()
end

-- Next letter which are found by search
function next(...)
	if(arg[1] == "-h") then
		print(help["next"])
		return
	end
	if(Search:hasNext()) then
		Search:next()
	else
		error(errors["nextNoMore"],1)
	end		
end

-- Previous letter which are found by search
function previous(...)
	if(arg[1] == "-h") then
		print(help["previous"])
		return
	end
	if(Search:hasPrevious()) then
		Search:previous()
	else
		error(errors["previousNoMore"],1)
	end

end

-- Select all found letters
function selectAll(...)
	if(arg[1] == "-h") then
		print(help["selectAll"])
		return
	end
	Search:selectAll()
end

-- show blocks
function show(...)
	if(arg[1] == "-h") then
		print(help["show"])
		return
	end
	Table:clear()
	Table:show(unpack(arg))	
end

-- add blocks
function add(...)
	if(arg[1] == "-h") then
		print(help["add"])
		return
	end
	if(arg["n"] < 1) then
		Table:clear()
	end
	Table:show(unpack(arg))	
end

-- show range
function showRange(a,b)
	if(a == "-h") then
		print(help["showRange"])
		return
	end
	Table:clear()
	Table:addRange(a,b)	
end

-- add range
function addRange(a,b)
	if(a == "-h") then
		print(help["addRange"])
		return
	end
	Table:addRange(a,b)	
end

-- Show only the found letters in table.
function addItems(...)
	if(arg[1] == "-h") then
		print(help["addItems"])
		return
	end
	
	Search:addItems()
end

-- Show only the found letters in table.
function showItems(...)
	if(arg[1] == "-h") then
		print(help["showItems"])
		return
	end
	
	Table:clear()
	Search:addItems()
end

-- Clear the table
function clear(...)
	if(arg[1] == "-h") then
		print(help["clear"])
		return
	end
	Table:clear()
end

-- overwrite the print function
function print(...)
	Output:print(unpack(arg))	
end

-- Show table
function searchTable(...)
	if(arg[1] == "-h") then
		print(help["searchTable"])
		return
	end
	if(arg["n"] > 1) then
		Search:searchTable(arg[1],arg[2])
	else
		Search:searchTable(arg[1])
	end

end

function addWeightRange(...)
		if(arg[1] == "-h") then
			print(help["addWeightRange"])
				  
			return
		end
	local err = Table:addWeightRange(unpack(arg))	
	if(err) then
		error(err,1)			
	end
end

function showWeightRange(...)
		if(arg[1] == "-h") then
			print(help["showWeightRange"])
				  
			return
		end
	Table:clear()
	local err = Table:addWeightRange(unpack(arg))	
	
	if(err) then
		error(err,1)			
	end
end

-- showGlyph
function showGlyph(a)
	if(a == "-h") then
		print(help["showGlyph"])
		return
	end
	showRange(a,a)
end

-- addGlyph
function addGlyph(a)
	if(a == "-h") then
		print(help["addGlyph"])
		return
	end
	addRange(a,a)
end


-- hideRange
function hideRange(a,b)
	if(a == "-h") then
		print(help["hideRange"])
		return
	end
	Table:hideRange(a,b)	
end

--hideGlyph
function hideGlyph(a)
	if(a == "-h") then
		print(help["hideGlyph"])
		return
	end
	hideRange(a,a)
end

--hide blocks
function hide(...)
	if(arg[1] == "-h") then
		print(help["hide"])
		return
	end
	Table:hide(unpack(arg))	
end

--Count of found items
function numberOfHits(...)
	if(arg[1] == "-h") then
		print(help["numberOfHits"])
		return
	end
	print(Search:numberOfHits()..messages["numberOfHits"])
end

--Jump to codepoint
function scrollTo(codepoint)
	if(codepoint == "-h") then
		print(help["scrollTo"])
		return
	end
	Search:search(codepoint,"c")
end

-- Show for Bachelor
function sort()
	Sort:sort()
end